module Md2site
  class HTMLUtils
    require "md2site/nkfguess"

    def initialize(htmlfname, mes)
      @htmlfname = htmlfname
      @mes = mes
    end

    def parse_line(state, line)
      event = get_event(line)

      @mes.output_debug("#{@lineno}|#{state}|#{event}|#{line}")

      case state
      when :IN_PART0
        next_state = state_in_part0(event, line)
      when :IN_PART1
        next_state = state_in_part1(event, line)
      when :IN_PART2
        next_state = state_in_part2(event, line)
      else
        @mes.output_fatal("state=#{state}")
        @mes.output_fatal("event=#{event}")
        raise
      end
      next_state
    end

    def divide_html
      part_struct = Struct.new(:lines, :status)
      @part0 = part_struct.new([], false)
      @part1 = part_struct.new([], false)
      @part2 = part_struct.new([], false)

      encoding = Nkfguess.guess_file(@htmlfname, @mes)
      mode = "r:#{encoding}"
      f = @mes.exc_file_gets(@htmlfname) { File.open(@htmlfname, mode) }

      @lines = []
      @lineno = 0
      state = :IN_PART0
      while (line = @mes.exc_file_gets(@htmlfname) { f.gets })
        state = parse_line(state, line)
      end
      @part2.lines << @lines

      @mes.exc_file_close(@htmlfname) { f.close }

      [@part0.lines.flatten.join("\n"),
       @part1.lines.flatten.join("\n"),
       @part2.lines.flatten.join("\n")]
    end

    private

    def get_event(line)
      line.chomp!
      begin
        ldc = line.downcase
      rescue Error => e
        @mes.output_exception(e)
        @mes.output_fatal("Can't convert downcase #{@htmlfname}")
        exit(100)
      rescue ArgumentError => e
        @mes.output_exception(e)
        @mes.output_fatal("Can't convert downcase #{@htmlfname}")
        exit(100)
      end
      @lineno += 1
      if ldc.index("<body")
        event = :BODY_BEGIN

      elsif ldc.index("</body>")
        event = :BODY_END

      else
        event = :ELSE
      end

      event
    end

    def state_in_part0(event, line)
      case event
      when :BODY_BEGIN
        @part0.lines << @lines
        @lines = []
        @lines << line
        next_state = :IN_PART1
      when :BODY_END
        @lines << line
        @part1.lines << @lines
        @lines = []
        next_state = :IN_PART2
      when :ELSE
        @lines << line
        next_state = :IN_PART0
      else
        raise
      end

      next_state
    end

    def state_in_part1(event, line)
      case event
      when :BODY_BEGIN
        @lines << line
        next_state = :IN_PART1
      when :BODY_END
        @lines << line
        @part1.lines << @lines
        @lines = []
        next_state = :IN_PART2
      when :ELSE
        @lines << line
        next_state = :IN_PART1
      else
        raise
      end

      next_state
    end

    def state_in_part2(event, line)
      case event
      when :BODY_BEGIN
        raise
      when :BODY_END
        raise
      when :ELSE
        @lines << line
        next_state = :IN_PART2
      else
        raise
      end

      next_state
    end
  end
end
